@@ -49,12 +49,18 @@ G_DEFINE_TYPE (McAccount, mc_account, G_TYPE_OBJECT);
 typedef struct
 {
   gchar *unique_name;
+  gchar *profile_name;
   GSList *display_names;
   GSList *normalized_names;
+  gboolean enabled;
 } McAccountPrivate;
 
 static gboolean mc_account_set_deleted (McAccount *account, gboolean deleted);
 static gboolean mc_account_is_deleted (McAccount *account);
+static gboolean _mc_account_gconf_get_boolean (McAccount *account,
+			const gchar *name, gboolean param, gboolean *value);
+static gboolean _mc_account_gconf_get_string (McAccount *account,
+			  const gchar *name, gboolean param, gchar **value);
 
 static void
 mc_account_finalize (GObject *object)
@@ -63,6 +69,7 @@ mc_account_finalize (GObject *object)
   McAccountPrivate *priv = MC_ACCOUNT_PRIV (self);
   
   g_free (priv->unique_name);
+  g_free (priv->profile_name);
   g_slist_foreach (priv->display_names, (GFunc)g_free, NULL);
   g_slist_free (priv->display_names);
   g_slist_foreach (priv->normalized_names, (GFunc)g_free, NULL);
@@ -84,13 +91,85 @@ mc_account_clear_cache (void)
 McAccount *
 _mc_account_new (const gchar *unique_name)
 {
+  McAccountPrivate *priv;
   McAccount *new;
+  gboolean enabled;
+  gchar *name;
+
   new = (McAccount *)g_object_new (MC_TYPE_ACCOUNT, NULL);
-  MC_ACCOUNT_PRIV (new)->unique_name = g_strdup (unique_name);
+  priv = MC_ACCOUNT_PRIV (new);
+  priv->unique_name = g_strdup (unique_name);
+
+  /* get enabledness status */
+  if (_mc_account_gconf_get_boolean (new, MC_ACCOUNTS_GCONF_KEY_ENABLED,
+				     FALSE, &enabled) && enabled)
+      priv->enabled = TRUE;
+
+  if (_mc_account_gconf_get_string (new, MC_ACCOUNTS_GCONF_KEY_NORMALIZED_NAME,
+				    FALSE, &name))
+      priv->normalized_names = g_slist_prepend (NULL, name);
+
+  if (_mc_account_gconf_get_string (new, MC_ACCOUNTS_GCONF_KEY_DISPLAY_NAME,
+				    FALSE, &name))
+      priv->display_names = g_slist_prepend (NULL, name);
 
   return new;
 }
 
+void
+_mc_account_set_enabled_priv (McAccount *account, gboolean enabled)
+{
+  g_return_if_fail (account != NULL);
+  MC_ACCOUNT_PRIV (account)->enabled = enabled;
+}
+
+static gint
+strcmp_null (gconstpointer a, gconstpointer b)
+{
+    if (a == b) return 0;
+    if (!a || !b) return 1;
+    return strcmp (a, b);
+}
+
+static GSList *
+set_first_element (GSList *list, const gchar *value)
+{
+    GSList *elem;
+
+    if ((elem = g_slist_find_custom(list, value, strcmp_null)) != NULL)
+    {
+	if (elem != list)
+	{
+	    /* move the new name at the beginning of the list */
+	    list = g_slist_remove_link (list, elem);
+	    list = g_slist_concat (elem, list);
+	}
+    }
+    else
+	list = g_slist_prepend (list, g_strdup (value));
+    return list;
+}
+
+void
+_mc_account_set_normalized_name_priv (McAccount *account, const gchar *name)
+{
+    McAccountPrivate *priv;
+
+    g_return_if_fail (account != NULL);
+    priv = MC_ACCOUNT_PRIV (account);
+    priv->normalized_names = set_first_element (priv->normalized_names, name);
+}
+
+void
+_mc_account_set_display_name_priv (McAccount *account, const gchar *name)
+{
+    McAccountPrivate *priv;
+
+    g_return_if_fail (account != NULL);
+    priv = MC_ACCOUNT_PRIV (account);
+    priv->display_names = set_first_element (priv->display_names, name);
+}
+
 static void
 mc_account_class_init (McAccountClass *klass)
 {
@@ -960,29 +1039,11 @@ const gchar *
 mc_account_get_normalized_name (McAccount *account)
 {
   McAccountPrivate *priv;
-  GSList *list;
-  gchar *name;
 
   g_return_val_if_fail (account != NULL, NULL);
   priv = MC_ACCOUNT_PRIV (account);
 
-  if (!_mc_account_gconf_get_string (account,
-				     MC_ACCOUNTS_GCONF_KEY_NORMALIZED_NAME,
-				     FALSE, &name))
-    return NULL;
-
-  if ((list = g_slist_find_custom(priv->normalized_names, name,
-				  (GCompareFunc)strcmp)) != NULL)
-  {
-      g_free (name);
-      name = list->data;
-  }
-  else
-  {
-      priv->normalized_names = g_slist_prepend (priv->normalized_names,
-						name);
-  }
-  return name;
+  return (priv->normalized_names) ? priv->normalized_names->data : NULL;
 }
 
 /**
@@ -997,9 +1058,15 @@ mc_account_get_normalized_name (McAccount *account)
 gboolean
 mc_account_set_normalized_name (McAccount *account, const gchar *name)
 {
-    return _mc_account_gconf_set_string (account,
+    if (_mc_account_gconf_set_string (account,
 				      MC_ACCOUNTS_GCONF_KEY_NORMALIZED_NAME,
-				      name);
+				      name))
+    {
+	_mc_account_set_normalized_name_priv (account, name);
+	return TRUE;
+    }
+    else
+	return FALSE;
 }
   
 /**
@@ -1029,22 +1096,21 @@ mc_account_get_unique_name (McAccount *account)
 McProfile *
 mc_account_get_profile (McAccount *account)
 {
-  McProfile *ret;
-  gchar *profile_name;
+  McAccountPrivate *priv;
 
   g_return_val_if_fail (account != NULL, NULL);
   g_return_val_if_fail (MC_ACCOUNT_PRIV (account)->unique_name != NULL,
                         NULL);
 
-  if (!_mc_account_gconf_get_string (account, MC_ACCOUNTS_GCONF_KEY_PROFILE,
-                                        FALSE, &profile_name))
-    return NULL;
+  priv = MC_ACCOUNT_PRIV (account);
+  if (G_UNLIKELY (!priv->profile_name))
+  {
+    if (!_mc_account_gconf_get_string (account, MC_ACCOUNTS_GCONF_KEY_PROFILE,
+				       FALSE, &priv->profile_name))
+      return NULL;
+  }
 
-  ret = mc_profile_lookup (profile_name);
-
-  g_free (profile_name);
-
-  return ret;
+  return mc_profile_lookup (priv->profile_name);
 }
 
 /**
@@ -1059,27 +1125,11 @@ const gchar *
 mc_account_get_display_name (McAccount *account)
 {
   McAccountPrivate *priv;
-  GSList *list;
-  gchar *name;
 
   g_return_val_if_fail (account != NULL, NULL);
   priv = MC_ACCOUNT_PRIV (account);
 
-  if (!_mc_account_gconf_get_string (account, MC_ACCOUNTS_GCONF_KEY_DISPLAY_NAME,
-                                        FALSE, &name))
-    return NULL;
-
-  if ((list = g_slist_find_custom(priv->display_names, name,
-				  (GCompareFunc)strcmp)) != NULL)
-  {
-      g_free (name);
-      name = list->data;
-  }
-  else
-  {
-      priv->display_names = g_slist_prepend (priv->display_names, name);
-  }
-  return name;
+  return (priv->display_names) ? priv->display_names->data : NULL;
 }
 
 /**
@@ -1095,9 +1145,15 @@ mc_account_get_display_name (McAccount *account)
 gboolean
 mc_account_set_display_name (McAccount *account, const gchar *name)
 {
-    return _mc_account_gconf_set_string (account,
-					 MC_ACCOUNTS_GCONF_KEY_DISPLAY_NAME,
-					 (name && *name) ? name : NULL);
+    if (_mc_account_gconf_set_string (account,
+				      MC_ACCOUNTS_GCONF_KEY_DISPLAY_NAME,
+				      (name && *name) ? name : NULL))
+    {
+	_mc_account_set_display_name_priv (account, name);
+	return TRUE;
+    }
+    else
+	return FALSE;
 }
 
 /**
@@ -1111,16 +1167,9 @@ mc_account_set_display_name (McAccount *account, const gchar *name)
 gboolean
 mc_account_is_enabled (McAccount *account)
 {
-  gboolean enabled;
-
   g_return_val_if_fail (account != NULL, FALSE);
-  g_return_val_if_fail (MC_ACCOUNT_PRIV (account)->unique_name != NULL, FALSE);
 
-  if (!_mc_account_gconf_get_boolean (account, MC_ACCOUNTS_GCONF_KEY_ENABLED,
-                                         FALSE, &enabled))
-    return FALSE;
-
-  return enabled;
+  return MC_ACCOUNT_PRIV (account)->enabled;
 }
 
 static gboolean
