@@ -143,6 +143,12 @@ struct presence_info
     gboolean allow_message;
 };
 
+struct param_data
+{
+    GSList *pr_params;
+    GHashTable *dest;
+};
+
 enum
 {
     PROP_0,
@@ -603,7 +609,7 @@ on_capabilities_changed (DBusGProxy *tp_conn_proxy,
     /* Return also if the "tp_chan_call" data is set (which means that a
      * request for this channel has already been made) */
     if (g_object_get_data (G_OBJECT (channel), "tp_chan_call") != NULL)
-	return;
+	goto done;
     chan_handle_type = mcd_channel_get_handle_type (channel);
     g_debug ("%s: requesting channel again (type = %s, handle_type = %u, handle = %u)",
 	     G_STRFUNC, chan_type, chan_handle_type, chan_handle);
@@ -614,6 +620,7 @@ on_capabilities_changed (DBusGProxy *tp_conn_proxy,
 					 mcd_async_request_chan_callback,
 					 channel);
     g_object_set_data (G_OBJECT (channel), "tp_chan_call", call);
+done:
     g_free (chan_type);
 }
 
@@ -942,7 +949,7 @@ _mcd_connection_setup_alias (McdConnectionPrivate *priv)
 	g_object_ref (priv->alias_proxy);
     }
     alias = mc_account_get_alias (priv->account);
-    if (!priv->alias || strcmp (priv->alias, alias) != 0)
+    if (alias && (!priv->alias || strcmp (priv->alias, alias) != 0))
     {
 	g_debug ("%s: setting alias '%s'", G_STRFUNC, alias);
 
@@ -1087,8 +1094,79 @@ mcd_async_connect_cb (DBusGProxy *proxy, GError *error, gpointer userdata)
     }
 }
 
+static GHashTable *
+get_extra_parameters (McdConnection *connection)
+{
+    McdMission *mission;
+ 
+    mission = mcd_mission_get_parent (MCD_MISSION (connection));
+    g_return_val_if_fail (mission != NULL, NULL);
+    mission = mcd_mission_get_parent (mission);
+    g_return_val_if_fail (mission != NULL && MCD_IS_MASTER (mission), NULL);
+    return mcd_master_get_connection_parameters (MCD_MASTER (mission));
+}
+
 static void
-mcd_connection_connect (McdConnection *connection, GHashTable *params)
+add_supported_param (gpointer key, gpointer value, gpointer userdata)
+{
+    struct param_data *pd = (struct param_data *)userdata;
+    gboolean found = FALSE;
+    GSList *list;
+ 
+    for (list = pd->pr_params; list != NULL; list = list->next)
+    {
+	McProtocolParam *param = list->data;
+	if (strcmp (param->name, key) == 0)
+	{
+	    found = TRUE;
+	    break;
+	}
+    }
+    if (found && !g_hash_table_lookup (pd->dest, key))
+    {
+	g_debug ("%s: adding parameter %s", G_STRFUNC, (gchar *)key);
+	g_hash_table_insert (pd->dest, key, value);
+    }
+}
+
+static void
+add_supported_extra_parameters (GHashTable *extra, McProfile *profile,
+				GHashTable *params)
+{
+    McProtocol *protocol;
+    struct param_data pd;
+    GSList *pr_params;
+
+    protocol = mc_profile_get_protocol (profile);
+    g_return_if_fail (protocol != NULL);
+
+    pr_params = mc_protocol_get_params (protocol);
+    pd.pr_params = pr_params;
+    pd.dest = params;
+    g_hash_table_foreach (extra, add_supported_param, &pd);
+    mc_protocol_free_params_list (pr_params);
+    g_object_unref (protocol);
+}
+
+static void
+remove_extra_parameter (gpointer key, gpointer value, gpointer userdata)
+{
+    GHashTable *params = (GHashTable *)userdata;
+    g_hash_table_steal (params, key);
+}
+
+static void
+remove_extra_parameters (GHashTable *extra_parameters, GHashTable *params)
+{
+    /* we need to remove the extra parameters from the parameters hash table
+     * because otherwise whey will be freed twice (and possibly in a wrong way)
+     * by the two g_hash_table_destroy() calls, since we didn't copy them */
+    g_hash_table_foreach (extra_parameters, remove_extra_parameter, params);
+    g_hash_table_destroy (extra_parameters);
+}
+
+static void
+_mcd_connection_connect (McdConnection *connection, GHashTable *params)
 {
     McdConnectionPrivate *priv = MCD_CONNECTION_PRIV (connection);
     TelepathyConnectionStatus conn_status;
@@ -1096,6 +1174,7 @@ mcd_connection_connect (McdConnection *connection, GHashTable *params)
     const gchar *protocol_name;
     const gchar *account_name;
     gchar *conn_bus_name, *conn_obj_path;
+    GHashTable *extra_parameters;
     GError *error = NULL;
     gboolean ret;
 
@@ -1106,20 +1185,26 @@ mcd_connection_connect (McdConnection *connection, GHashTable *params)
     g_debug ("%s: Trying connect account: %s",
 	     G_STRFUNC, (gchar *) account_name);
 
+    extra_parameters = get_extra_parameters (connection);
+    add_supported_extra_parameters (extra_parameters, profile, params);
     ret = tp_connmgr_request_connection (DBUS_G_PROXY (priv->tp_conn_mgr),
 					 protocol_name, params,
 					 &conn_bus_name, &conn_obj_path,
 					 &error);
+    remove_extra_parameters (extra_parameters, params);
     g_object_unref (profile);
     if (!ret)
     {
-	g_warning ("%s: tp_connmgr_request_connection failed: %s",
-		   G_STRFUNC, error->message);
+	if (error)
+	{
+	    g_warning ("%s: tp_connmgr_request_connection failed: %s",
+		       G_STRFUNC, error->message);
+	    g_error_free (error);
+	}
 	mcd_presence_frame_set_account_status (priv->presence_frame,
 					       priv->account,
 					       TP_CONN_STATUS_DISCONNECTED,
 					       TP_CONN_STATUS_REASON_NETWORK_ERROR);
-	g_error_free (error);
 	return;
     }
 
@@ -1190,7 +1275,7 @@ provisioning_cb (McdProvisioning *prov, GHashTable *parameters, GError *error,
 					       TP_CONN_STATUS_REASON_AUTHENTICATION_FAILED);
 	return;
     }
-    mcd_connection_connect (connection, parameters);
+    _mcd_connection_connect (connection, parameters);
     g_hash_table_destroy (parameters);
 }
 
@@ -1270,7 +1355,7 @@ mcd_connection_get_params_and_connect (McdConnection *connection)
     if (!requesting_provisioning)
     {
 	params = mc_account_get_params (priv->account);
-	mcd_connection_connect (connection, params);
+	_mcd_connection_connect (connection, params);
 	g_hash_table_destroy (params);
     }
     g_object_unref (profile);
@@ -1544,7 +1629,6 @@ _mcd_connection_set_property (GObject * obj, guint prop_id,
 	if (priv->account)
 	    g_object_unref (priv->account);
 	priv->account = account;
-	_mcd_connection_setup (MCD_CONNECTION (obj));
 	break;
     default:
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, prop_id, pspec);
@@ -1761,15 +1845,15 @@ map_tp_error_to_mc_error (McdChannel *channel, GError *tp_error)
     {
 	mc_error_code = MC_CONTACT_DOES_NOT_SUPPORT_VOICE_ERROR;
     }
-    else if (dbus_g_error_has_name(tp_error, "org.freedesktop.Telepathy.Error.ChannelBanned"))
+    else if (dbus_g_error_has_name(tp_error, "org.freedesktop.Telepathy.Error.Channel.Banned"))
     {
 	mc_error_code = MC_CHANNEL_BANNED_ERROR;
     }
-    else if (dbus_g_error_has_name(tp_error, "org.freedesktop.Telepathy.Error.ChannelFull"))
+    else if (dbus_g_error_has_name(tp_error, "org.freedesktop.Telepathy.Error.Channel.Full"))
     {
 	mc_error_code = MC_CHANNEL_FULL_ERROR;
     }
-    else if (dbus_g_error_has_name(tp_error, "org.freedesktop.Telepathy.Error.ChannelInviteOnly"))
+    else if (dbus_g_error_has_name(tp_error, "org.freedesktop.Telepathy.Error.Channel.InviteOnly"))
     {
 	mc_error_code = MC_CHANNEL_INVITE_ONLY_ERROR;
     }
@@ -2327,3 +2411,15 @@ mcd_connection_restart (McdConnection *connection)
 	tp_conn_disconnect (DBUS_G_PROXY (priv->tp_conn), NULL);
 }
 
+/**
+ * mcd_connection_connect:
+ * @connection: the #McdConnection.
+ *
+ * Activate @connection.
+ */
+void
+mcd_connection_connect (McdConnection *connection)
+{
+    _mcd_connection_setup (connection);
+}
+
